var Kafka, KafkaProducers, each, uuid,
  bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

each = require('async/eachOf');

uuid = require('uuid/v4');

Kafka = require('node-rdkafka');

KafkaProducers = (function() {
  function KafkaProducers(arg) {
    var brokers, i, len, producer, topic, topicName, topics;
    brokers = arg.brokers, topics = arg.topics;
    this.produce = bind(this.produce, this);
    this.connect = bind(this.connect, this);
    if (!Array.isArray(topics)) {
      topics = [topics];
    }
    this.producers = {};
    for (i = 0, len = topics.length; i < len; i++) {
      topicName = topics[i];
      producer = new Kafka.Producer({
        'metadata.broker.list': brokers
      });
      topic = Kafka.Topic(topicName);
      producer.on('error', function(err) {
        throw err;
      });
      this.producers[topicName] = {
        partition: null,
        producer: producer,
        topic: topic
      };
    }
  }

  KafkaProducers.prototype.connect = function(callback) {
    var handlers, middleware;
    handlers = {};
    middleware = {};
    return each(this.producers, (function(_this) {
      return function(arg, topicName, done) {
        var partition, producer, topic;
        producer = arg.producer, topic = arg.topic, partition = arg.partition;
        producer.on('event.error', callback);
        producer.on('ready', function() {
          var mw;
          mw = _this._generic_mw(producer, topic, partition);
          handlers[topicName] = [mw, _this._generic_response];
          middleware[topicName] = mw;
          return done();
        });
        return producer.connect();
      };
    })(this), function(err) {
      return callback(err, handlers, middleware);
    });
  };

  KafkaProducers.prototype._generic_mw = function(producer, topic, partition) {
    return this._with_message(function(msg, req, res, next) {
      var ex;
      try {
        producer.produce(topic, partition, Buffer.from(JSON.stringify(msg)), msg.key);
        return next();
      } catch (error) {
        ex = error;
        return next(ex);
      }
    });
  };

  KafkaProducers.prototype._generic_response = function(req, res) {
    return res.sendStatus(202);
  };

  KafkaProducers.prototype.produce = function(topicName, msg, key) {
    var ex, partition, producer, ref, topic;
    if (key == null) {
      key = uuid();
    }
    if (!(arguments.length > 1)) {
      return console.log("Topic name and msg payload are both required");
    }
    if (this.producers[topicName] == null) {
      return console.log("No topic " + topicName + " found");
    }
    ref = this.producers[topicName], producer = ref.producer, topic = ref.topic, partition = ref.partition;
    try {
      return producer.produce(topic, partition, Buffer.from(JSON.stringify(msg)), key);
    } catch (error) {
      ex = error;
      return console.log(ex);
    }
  };

  KafkaProducers.prototype._with_message = function(callback) {
    return function(req, res, next) {
      var msg, ref, ref1;
      msg = {};
      msg.http_method = req.method.toUpperCase();
      msg.key = uuid();
      msg.timestamp = new Date().toISOString();
      msg.user = ((ref = res.locals) != null ? ref.user : void 0) || req.user || {};
      msg.payload = ((ref1 = res.locals) != null ? ref1.body : void 0) || req.body || {};
      res.locals.message = msg;
      return callback(msg, req, res, next);
    };
  };

  return KafkaProducers;

})();

module.exports = KafkaProducers;
